/* *************************************************************************
 * function TOPZ
 * DETERMINES FIRST VALID DEPTH FOR A VALID VARIABLE OF A CAST
 * INPUTS:
 * numz = number of samples in a cast         (int)
 * val  = array of values of size numz        (float)
 * z    = array of depths of size numz        (float)
 *        (depth increases downward (non-negative))
 * OUTPUT:
 * surface value (float) for a variable
 * ************************************************************************* */

float TOPZ(numz,val,z,missing)
int	numz; 
float missing,val[],z[]; 
{
	int	n; 
	float shallow=-99.99;

/* ************************************************************************
 * FIND SHALLOWEST DEPTH WITH VALID VALUE IN CAST
 * ************************************************************************ */
   
	if (val[0] != missing) shallow=z[0];
	for (n=1; n<numz; n++)
		if ((z[n] < shallow) && (val[n] != missing))
			shallow=z[n];
	return(shallow);
}

/* *************************************************************************
 * function SURF
 * DETERMINES FIRST VALID SURFACE VALUE FOR A VARIABLE OF A CAST
 * INPUTS:
 * numz = number of samples in a cast         (int)
 * val  = array of values of size numz        (float)
 * z    = array of depths of size numz        (float)
 * maxz = maximum depth considered "surface"  (float)
 * OUTPUT:
 * surface value (float) for a variable
 * ************************************************************************* */

float SURF(numz,val,z,maxz)
int	numz; 
float val[],z[],maxz; 
{
	int	n; 
	float surfv=-99.99;

/* ************************************************************************
 * FIND SURFACE VALUE IF ONE EXISTS ABOVE maxz
 * ************************************************************************ */
	for (n=0; n<numz; n++)
		if ((z[n] <= maxz) && (surfv<=0.0))
			surfv=val[n];
	return(surfv);
}

/* *************************************************************************
 * function COUNT
 * DETERMINES NUMBER OF TIMES A VARIABLE WAS MEASURED 
 * WITHIN THE MIXED LAYER
 * INPUTS:
 * numz = number of samples in a cast  (int)
 * val  = array of values of size numz (float)
 * z    = array of depths of size numz (float)
 * mix  = mixed layer depth            (float)
 * OUTPUT
 * number of valid values (int) within the mixed layer for a variable
 * ************************************************************************* */

int COUNT(numz,val,z,mix)
int	numz; 
float val[],z[]; 
float mix;
{
	int	n; 
	int	cntv=0;

	for (n=0; n<numz; n++) 
		if ((z[n] < mix) && (val[n]>=-5.0))
			cntv++;
	return(cntv);
}

/* *************************************************************************
 * function INTEGRATE
 * DETERMINES MIXED LAYER INTEGRATION OF A VARIABLE USING ALL SAMPLES
 * WITHIN THE MIXED LAYER
 * INPUTS: 
 * numz = number of samples in a cast  (int)
 * val  = array of values of size numz (float)
 * z    = array of depths of size numz (float)
 * mix  = mixed layer depth            (float)
 * OUTPUT
 * integrated value of variable (float) is returned 
 * using scheme diagrammed below:
 * *************************************************************************
 * INTEGRATE VARIABLE WITHIN MLD - if surface value in < 10 m water
 *
 *    surf (z0) ------------------------------------
 *                    | s1        * (z1-z0)
 *    z1 s1        x
 *                    | (s1+s2)/2 * (z2-z1)
 *    z2 s2        x
 *                    | (s2+s3)/2 * (z3-z2)
 *                    |
 *    z3 s3        x
 *                    | s3        * (mld-z3)
 *                    |
 *    mld       ///////////////////////////////////
 *                    |
 *    z4 s4        x
 * ************************************************************************* */

float INTEGRATE(numz,val,z,mix)
int   numz;
float mix;
float val[],z[];
{
   int   n,nn;
   float lastz=0.0, lastv=0.0, range=0.0;
   float totv=0.0;
	float	tmpz, tmpval;

/* ************************************************************************
 * MAKE ADJUSTMENT WHERE "DEEPER" PRESSURE < "SHALLOWER" PRESSURE
 * ************************************************************************ */
	for (nn=0; nn<2; nn++)
   	for (n=0; n<numz-1; n++) 
			if (z[n]>z[n+1]) {
				tmpz=z[n];
				z[n]=z[n+1];
				z[n+1]=tmpz;
				tmpval=val[n];
				val[n]=val[n+1];
				val[n+1]=tmpval;
			}

   for (n=0; n<numz; n++) {
/* ************************************************************************
 * FIND NEXT SAMPLE DEPTH WHERE VALID MEASUREMENT
 * ************************************************************************ */
      if ((val[n]>=-5.0) && (mix>=0.0)) {
         if (n==0) lastz=0.0;
         else for (nn=1; n-nn>=0; nn++) {
            if (z[n-nn]>=0.0) {
               if (nn==n) {
                  lastz=0.0;
                  lastv=val[n];
               }
               if (val[n-nn]>=-5.0) {
                  lastz=z[n-nn];
                  lastv=val[n-nn];
                  nn=n;
               }
            }
         }
/* *************************************************************************
 * INTEGRATE VARIABLE WITHIN MLD - if surface value in < 10 m water
 *
 *    surf (z0) ------------------------------------
 *                    | s1        * (z1-z0)
 *    z1 s1        x
 *                    | (s1+s2)/2 * (z2-z1)
 *    z2 s2        x
 *                    | (s2+s3)/2 * (z3-z2)
 *                    |
 *    z3 s3        x
 *                    | s3        * (mld-z3)
 *                    |
 *    mld       ///////////////////////////////////
 *                    |
 *    z4 s4        x
 * ************************************************************************* */

         if (n==0) {
            range=z[n]-lastz;
            totv += val[n]*range;
            if ((n+1<numz) && (z[n+1]>mix)) {
               range=mix-z[n];
               totv += val[n]*range;
            }
         }

         else if (z[n]<=mix)  {
            range=z[n]-lastz;
            totv += ((val[n]+lastv)/2)*range;
            if ((n+1<numz) && (z[n+1]>mix)) {
               range=mix-z[n];
               totv += val[n]*range;
            }
         }
      }
   }

/* ************************************************************************
 * MAKE ADJUSTMENT WHERE LAST VALUE IS SHALLOWER THAN MLD
 * ************************************************************************ */
	if (z[numz-1]<mix) {
		range=mix-z[numz-1];
		totv += val[numz-1]*range;
	}
   return(totv);
}

/* *************************************************************************
 * function YDAY
 * CALCULATES YEAR-DAY FOR MOST JGOFS EVENT NUMBERS (where format: MMDDHHmm)
 * INPUTS:
 * year  year in 4 digits, YYYY  (int)
 * event event number, MMDDHHmm  (int)
 * OUTPUT:
 * year-day as DDD.ddd (day-of-year) (float)
 * ************************************************************************* */
float YDAY(year,event)
int   year,event;
{
/* year =  YYYY
 * event=  a UTC (GMT) date and time designation in the form MMDDHHmm that
 *         indicates when the event occurred.  Generally one event began as the
 *         preceding event ended except for transit time between stations or
 *         after long net tows when the ship repositioned.
 */
   int   month, day;
   int   nday1[]={0,31,59,90,120,151,181,212,243,273,304,334}; /* non-leap */
   int   nday2[]={0,31,60,91,121,152,182,213,244,274,305,335}; /* leap */
   float hour,YDAY;

   month  =event/1e6;
   day    =(event - (month*1e6))/1e4;
   hour   =(event - ((month*1e6)+(day*1e4)))/100;
   hour   =(int)hour + ((hour-(int)hour)/.60);
   if (((year%4 == 0) && (year%100 != 0)) || (year%400==0))
	   YDAY   =nday2[month-1] + day + (hour/24);
	else
	   YDAY   =nday1[month-1] + day + (hour/24);
   return (YDAY);
}

/* *************************************************************************
 * function YYMMDD2YDAY
 * DETERMINES YEAR-DAY FROM DATE (YYMMDD)
 * ************************************************************************* */
float YYMMDD2YDAY(yymmdd)
int   yymmdd;
{

   int   year;
   int   nday1[]={0,31,59,90,120,151,181,212,243,273,304,334}; /* non-leap */
   int   nday2[]={0,31,60,91,121,152,182,213,244,274,305,335}; /* leap */
   float YDAY;

   year=1900+(yymmdd/10000);
   if (((year%4 == 0) && (year%100 != 0)) || (year%400==0))
      YDAY   =nday2[((yymmdd%10000)/100)-1] + (yymmdd%100);
   else
      YDAY   =nday1[((yymmdd%10000)/100)-1] + (yymmdd%100);
   return (YDAY);
}

/* *************************************************************************
 * function YDAY2YYMMDD
 * DETERMINES DATE FROM YEAR (YYYY) and YEAR-DAY (DDD)
 * ************************************************************************* */
int YDAY2YYMMDD(year,yday)
int year, yday; 
year =  YYYY
yday =  DDD 
{
   int   m=0;
	int	yymmdd;
   int   nday1[]={0,31,59,90,120,151,181,212,243,273,304,334,365}; /* non-leap */
   int   nday2[]={0,31,60,91,121,152,182,213,244,274,305,335,366}; /* leap */

   if (((year%4 == 0) && (year%100 != 0)) || (year%400==0)) {
		while (yday>nday2[m]) m++;
		yymmdd=((year%100)*10000) + (m*100) + (yday-nday2[m-1]);
	}
   else {
		while (yday>nday1[m]) m++;
		yymmdd=((year%100)*10000) + (m*100) + (yday-nday1[m-1]);
	}
   return (yymmdd);
}

/* *************************************************************************
 * function MLD
 * DETERMINES MIXED LAYER DEPTH FOR A GIVEN CAST BASED ON DENSITY
 * (this function assumes no missing values in val or z arrays)
 * INPUTS:
 * numz = number of samples in a cast         (int)
 * val  = array of sigma of size numz         (float)
 * z    = array of depths of size numz        (float)
 * diff = criterion value - diff from surface (float)
 *  e.g.  0.125 (sigma)
 * OUTPUT:
 * mixed layer depth (float) for the cast
 * ************************************************************************* */

float MLD(numz,val,z,diff,maxz)
int   numz;
float val[],z[],diff,maxz;
{
   int   n,done=0;
   float mld=-99.99, crit=-99.99;

  /* -----------------------------------------------------
   * USING SHALLOWEST VALID SIGMA VALUE ABOVE maxz
   * DETERMINE VALUE OF crit BY ADDING diff TO THAT VALUE 
   * ----------------------------------------------------- */
   for (n=0; n<numz; n++)
      if ((crit < 0.) && (z[n] >=0.) && (z[n] <= maxz))
         crit = val[n]+diff;

  /* -----------------------------------------------------
   * FIND BOUNDING DEPTHS FOR crit, and 
   * INTERPOLATE DEPTH OF crit TO GET mld 
   * ----------------------------------------------------- */
	if (crit > 0)
	   for (n=0; n<numz-1 && done<1; n++) 
	      if ((val[n] < crit) && (val[n+1] >= crit)) {
           	mld = z[n] + (((crit-val[n]) / (val[n+1]-val[n])) *
                    (z[n+1] - z[n]));
				done=1;
			}
   return(mld);
}

/* *************************************************************************
 * function MLD_T
 * DETERMINES MIXED LAYER DEPTH FOR A GIVEN CAST BASED ON TEMPERATURE
 * INPUTS:
 * numz = number of samples in a cast         (int)
 * val  = array of temp of size numz          (float)
 * z    = array of depths of size numz        (float)
 * diff = criterion value - diff from surface (float)
 *  e.g. 0.5 (deg) or 0.125 (sigma)
 * maxz = maximum depth considered "surface"  (float)
 * OUTPUT:
 * mixed layer depth (float) for the cast
 * ************************************************************************* */

float MLD_T(numz,val,z,diff,maxz)
int   numz;
float val[],z[],diff,maxz;
{
   int   n;
   float mld=-99.99, crit=-99.99;

   for (n=0; n<numz; n++)
      if ((crit < 0.) && (val[n]>=0.0) &&
          (z[n] >=0.0) && (z[n] <= maxz))
         crit = val[n]-diff;

   for (n=0; n<numz-1; n++)
      if (val[n] > crit) {
         if (val[n+1]<=crit)
            mld = z[n] +
                  (((crit-val[n]) / (val[n+1]-val[n])) *
                    (z[n+1] - z[n]));
         else if ((val[n+2] <= crit) && ((n+2)<numz))
            mld = z[n] +
                  (((crit-val[n]) / (val[n+2]-val[n])) *
                    (z[n+2] - z[n]));
      }
   return(mld);
}

/* *************************************************************************
 * GENERAL PURPOSE SWAP ROUTINE
 * syntx is swapf(&val1,&val2)
 *
 * included here because mixed layer depth routines often
 * require sorting samples within a cast from shallowest to deepest.
 * ************************************************************************* */
void swapf(a,b)
float *a,*b;
{
   float t;
   t=*a;
   *a=*b;
   *b=t;
}
